@@ -1,11 +1,18 @@
 package app
 
 import (
+	"context"
 	"fmt"
+	"os"
+	"strings"
 
+	"github.com/abiosoft/colima/cli"
 	"github.com/abiosoft/colima/config"
+	"github.com/abiosoft/colima/config/configmanager"
 	"github.com/abiosoft/colima/environment"
+	"github.com/abiosoft/colima/environment/container/docker"
 	"github.com/abiosoft/colima/environment/container/kubernetes"
+	"github.com/abiosoft/colima/environment/container/ubuntu"
 	"github.com/abiosoft/colima/environment/host"
 	"github.com/abiosoft/colima/environment/vm/lima"
 	log "github.com/sirupsen/logrus"
@@ -14,9 +21,9 @@ import (
 type App interface {
 	Active() bool
 	Start(config.Config) error
-	Stop() error
+	Stop(force bool) error
 	Delete() error
-	SSH(...string) error
+	SSH(layer bool, args ...string) error
 	Status() error
 	Version() error
 	Runtime() (string, error)
@@ -42,8 +49,16 @@ type colimaApp struct {
 }
 
 func (c colimaApp) Start(conf config.Config) error {
-	log.Println("starting", config.Profile().DisplayName)
+	ctx := context.WithValue(context.Background(), config.CtxKey(), conf)
 
+	{
+		runtime := conf.Runtime
+		if conf.Kubernetes.Enabled {
+			runtime += "+k3s"
+		}
+		log.Println("starting", config.CurrentProfile().DisplayName)
+		log.Println("runtime:", runtime)
+	}
 	var containers []environment.Container
 	// runtime
 	{
@@ -53,7 +68,7 @@ func (c colimaApp) Start(conf config.Config) error {
 		}
 		containers = append(containers, env)
 	}
-	// kubernetes
+	// kubernetes should come after required runtime
 	if conf.Kubernetes.Enabled {
 		env, err := c.containerEnvironment(kubernetes.Name)
 		if err != nil {
@@ -61,47 +76,55 @@ func (c colimaApp) Start(conf config.Config) error {
 		}
 		containers = append(containers, env)
 	}
+	// ubuntu layer should come last
+	if conf.Layer {
+		env, err := c.containerEnvironment(ubuntu.Name)
+		if err != nil {
+			return err
+		}
+		containers = append(containers, env)
+	}
 
 	// the order for start is:
 	//   vm start -> container runtime provision -> container runtime start
 
 	// start vm
-	if err := c.guest.Start(conf); err != nil {
+	if err := c.guest.Start(ctx, conf); err != nil {
 		return fmt.Errorf("error starting vm: %w", err)
 	}
 
-	// persist runtime for future reference.
-	if err := c.setRuntime(conf.Runtime); err != nil {
-		return fmt.Errorf("error setting current runtime: %w", err)
-	}
-	// persist kubernetes version for future reference.
-	if err := c.setKubernetesVersion(conf.Kubernetes.Version); err != nil {
-		return fmt.Errorf("error setting kubernetes version: %w", err)
-	}
-
 	// provision and start container runtimes
 	for _, cont := range containers {
-		if err := cont.Provision(); err != nil {
+		log := log.WithField("context", cont.Name())
+		log.Println("provisioning ...")
+		if err := cont.Provision(ctx); err != nil {
 			return fmt.Errorf("error provisioning %s: %w", cont.Name(), err)
 		}
-		if err := cont.Start(); err != nil {
+		log.Println("starting ...")
+		if err := cont.Start(ctx); err != nil {
 			return fmt.Errorf("error starting %s: %w", cont.Name(), err)
 		}
 	}
 
+	// persist the current runtime
+	if err := c.setRuntime(conf.Runtime); err != nil {
+		log.Error(fmt.Errorf("error persisting runtime settings: %w", err))
+	}
+
 	log.Println("done")
 	return nil
 }
 
-func (c colimaApp) Stop() error {
-	log.Println("stopping", config.Profile().DisplayName)
+func (c colimaApp) Stop(force bool) error {
+	ctx := context.Background()
+	log.Println("stopping", config.CurrentProfile().DisplayName)
 
 	// the order for stop is:
 	//   container stop -> vm stop
 
-	// stop container runtimes
-	if c.guest.Running() {
-		containers, err := c.currentContainerEnvironments()
+	// stop container runtimes if not a forceful shutdown
+	if c.guest.Running(ctx) && !force {
+		containers, err := c.currentContainerEnvironments(ctx)
 		if err != nil {
 			log.Warnln(fmt.Errorf("error retrieving runtimes: %w", err))
 		}
@@ -109,7 +132,11 @@ func (c colimaApp) Stop() error {
 		// stop happens in reverse of start
 		for i := len(containers) - 1; i >= 0; i-- {
 			cont := containers[i]
-			if err := cont.Stop(); err != nil {
+
+			log := log.WithField("context", cont.Name())
+			log.Println("stopping ...")
+
+			if err := cont.Stop(ctx); err != nil {
 				// failure to stop a container runtime is not fatal
 				// it is only meant for graceful shutdown.
 				// the VM will shut down anyways.
@@ -120,7 +147,7 @@ func (c colimaApp) Stop() error {
 
 	// stop vm
 	// no need to check running status, it may be in a state that requires stopping.
-	if err := c.guest.Stop(); err != nil {
+	if err := c.guest.Stop(ctx, force); err != nil {
 		return fmt.Errorf("error stopping vm: %w", err)
 	}
 
@@ -129,7 +156,8 @@ func (c colimaApp) Stop() error {
 }
 
 func (c colimaApp) Delete() error {
-	log.Println("deleting", config.Profile().DisplayName)
+	ctx := context.Background()
+	log.Println("deleting", config.CurrentProfile().DisplayName)
 
 	// the order for teardown is:
 	//   container teardown -> vm teardown
@@ -139,13 +167,17 @@ func (c colimaApp) Delete() error {
 	// it is thereby necessary to teardown containers as well.
 
 	// teardown container runtimes
-	if c.guest.Running() {
-		containers, err := c.currentContainerEnvironments()
+	if c.guest.Running(ctx) {
+		containers, err := c.currentContainerEnvironments(ctx)
 		if err != nil {
 			log.Warnln(fmt.Errorf("error retrieving runtimes: %w", err))
 		}
 		for _, cont := range containers {
-			if err := cont.Teardown(); err != nil {
+
+			log := log.WithField("context", cont.Name())
+			log.Println("deleting ...")
+
+			if err := cont.Teardown(ctx); err != nil {
 				// failure here is not fatal
 				log.Warnln(fmt.Errorf("error during teardown of %s: %w", cont.Name(), err))
 			}
@@ -153,12 +185,12 @@ func (c colimaApp) Delete() error {
 	}
 
 	// teardown vm
-	if err := c.guest.Teardown(); err != nil {
+	if err := c.guest.Teardown(ctx); err != nil {
 		return fmt.Errorf("error during teardown of vm: %w", err)
 	}
 
 	// delete configs
-	if err := config.Teardown(); err != nil {
+	if err := configmanager.Teardown(); err != nil {
 		return fmt.Errorf("error deleting configs: %w", err)
 	}
 
@@ -166,30 +198,73 @@ func (c colimaApp) Delete() error {
 	return nil
 }
 
-func (c colimaApp) SSH(args ...string) error {
-	if !c.guest.Running() {
-		return fmt.Errorf("%s not running", config.Profile().DisplayName)
+func (c colimaApp) SSH(layer bool, args ...string) error {
+	ctx := context.Background()
+	if !c.guest.Running(ctx) {
+		return fmt.Errorf("%s not running", config.CurrentProfile().DisplayName)
 	}
 
-	return c.guest.RunInteractive(args...)
+	if !layer {
+		return c.guest.RunInteractive(args...)
+	}
+
+	conf, err := lima.InstanceConfig()
+	if err != nil {
+		return err
+	}
+	if !conf.Layer {
+		return c.guest.RunInteractive(args...)
+	}
+
+	resp, err := lima.ShowSSH(config.CurrentProfile().ID, layer, "args")
+	if err != nil {
+		return fmt.Errorf("error getting ssh config: %w", err)
+	}
+	if !resp.Layer {
+		return c.guest.RunInteractive(args...)
+	}
+
+	cmdArgs := resp.Output
+
+	wd, err := os.Getwd()
+	if err != nil {
+		log.Debug(fmt.Errorf("cannot get working dir: %w", err))
+	}
+
+	if len(args) > 0 {
+		args = append([]string{"-q", "-t", resp.IPAddress, "--"}, args...)
+	} else if wd != "" {
+		args = []string{"-q", "-t", resp.IPAddress, "--", "cd " + wd + " 2> /dev/null; bash --login"}
+	}
+
+	args = append(strings.Fields(cmdArgs), args...)
+	return cli.CommandInteractive("ssh", args...).Run()
+
 }
 
 func (c colimaApp) Status() error {
-	if !c.guest.Running() {
-		return fmt.Errorf("%s is not running", config.Profile().DisplayName)
+	ctx := context.Background()
+	if !c.guest.Running(ctx) {
+		return fmt.Errorf("%s is not running", config.CurrentProfile().DisplayName)
 	}
 
-	currentRuntime, err := c.currentRuntime()
+	currentRuntime, err := c.currentRuntime(ctx)
 	if err != nil {
 		return err
 	}
 
-	log.Println(config.Profile().DisplayName, "is running")
-	log.Println("runtime:", currentRuntime)
+	log.Println(config.CurrentProfile().DisplayName, "is running")
 	log.Println("arch:", c.guest.Arch())
+	log.Println("runtime:", currentRuntime)
+	if conf, err := lima.InstanceConfig(); err == nil {
+		log.Println("mountType:", conf.MountType)
+	}
+	if currentRuntime == docker.Name {
+		log.Println("socket:", "unix://"+docker.HostSocketFile())
+	}
 
 	// kubernetes
-	if k, err := c.Kubernetes(); err == nil && k.Running() {
+	if k, err := c.Kubernetes(); err == nil && k.Running(ctx) {
 		log.Println("kubernetes: enabled")
 	}
 
@@ -197,39 +272,44 @@ func (c colimaApp) Status() error {
 }
 
 func (c colimaApp) Version() error {
-	if !c.guest.Running() {
+	ctx := context.Background()
+	if !c.guest.Running(ctx) {
 		return nil
 	}
 
-	containerRuntimes, err := c.currentContainerEnvironments()
+	containerRuntimes, err := c.currentContainerEnvironments(ctx)
 	if err != nil {
 		return err
 	}
 
 	var kube environment.Container
 	for _, cont := range containerRuntimes {
-		if cont.Name() == kubernetes.Name {
+		switch cont.Name() {
+		case kubernetes.Name:
 			kube = cont
 			continue
+		case ubuntu.Name:
+			continue
 		}
+
 		fmt.Println()
 		fmt.Println("runtime:", cont.Name())
 		fmt.Println("arch:", c.guest.Arch())
-		fmt.Println(cont.Version())
+		fmt.Println(cont.Version(ctx))
 	}
 
-	if kube != nil && kube.Version() != "" {
+	if kube != nil && kube.Version(ctx) != "" {
 		fmt.Println()
 		fmt.Println(kubernetes.Name)
-		fmt.Println(kube.Version())
+		fmt.Println(kube.Version(ctx))
 	}
 
 	return nil
 }
 
-func (c colimaApp) currentRuntime() (string, error) {
-	if !c.guest.Running() {
-		return "", fmt.Errorf("%s is not running", config.Profile().DisplayName)
+func (c colimaApp) currentRuntime(ctx context.Context) (string, error) {
+	if !c.guest.Running(ctx) {
+		return "", fmt.Errorf("%s is not running", config.CurrentProfile().DisplayName)
 	}
 
 	r := c.guest.Get(environment.ContainerRuntimeKey)
@@ -244,16 +324,12 @@ func (c colimaApp) setRuntime(runtime string) error {
 	return c.guest.Set(environment.ContainerRuntimeKey, runtime)
 }
 
-func (c colimaApp) setKubernetesVersion(version string) error {
-	return c.guest.Set(environment.KubernetesVersionKey, version)
-}
-
-func (c colimaApp) currentContainerEnvironments() ([]environment.Container, error) {
+func (c colimaApp) currentContainerEnvironments(ctx context.Context) ([]environment.Container, error) {
 	var containers []environment.Container
 
 	// runtime
 	{
-		runtime, err := c.currentRuntime()
+		runtime, err := c.currentRuntime(ctx)
 		if err != nil {
 			return nil, err
 		}
@@ -265,10 +341,15 @@ func (c colimaApp) currentContainerEnvironments() ([]environment.Container, erro
 	}
 
 	// detect and add kubernetes
-	if k, err := c.containerEnvironment(kubernetes.Name); err == nil && k.Running() {
+	if k, err := c.containerEnvironment(kubernetes.Name); err == nil && k.Running(ctx) {
 		containers = append(containers, k)
 	}
 
+	// detect and add ubuntu layer
+	if u, err := c.containerEnvironment(ubuntu.Name); err == nil && u.Running(ctx) {
+		containers = append(containers, u)
+	}
+
 	return containers, nil
 }
 
@@ -285,7 +366,7 @@ func (c colimaApp) containerEnvironment(runtime string) (environment.Container,
 }
 
 func (c colimaApp) Runtime() (string, error) {
-	return c.currentRuntime()
+	return c.currentRuntime(context.Background())
 }
 
 func (c colimaApp) Kubernetes() (environment.Container, error) {
@@ -293,5 +374,5 @@ func (c colimaApp) Kubernetes() (environment.Container, error) {
 }
 
 func (c colimaApp) Active() bool {
-	return c.guest.Running()
+	return c.guest.Running(context.Background())
 }
