@@ -1,6 +1,7 @@
 package cmd
 
 import (
+	"encoding/json"
 	"fmt"
 	"text/tabwriter"
 
@@ -11,6 +12,10 @@ import (
 	"github.com/spf13/cobra"
 )
 
+var listCmdArgs struct {
+	json bool
+}
+
 // listCmd represents the version command
 var listCmd = &cobra.Command{
 	Use:     "list",
@@ -26,21 +31,36 @@ A new instance can be created during 'colima start' by specifying the '--profile
 			return err
 		}
 
+		if listCmdArgs.json {
+			encoder := json.NewEncoder(cmd.OutOrStdout())
+			// print instance per line to conform with Lima's output
+			for _, instance := range instances {
+				// dir should be hidden from the output
+				instance.Dir = ""
+				if err := encoder.Encode(instance); err != nil {
+					return err
+				}
+			}
+			return nil
+		}
+
 		w := tabwriter.NewWriter(cmd.OutOrStdout(), 4, 8, 4, ' ', 0)
-		fmt.Fprintln(w, "PROFILE\tSTATUS\tARCH\tCPUS\tMEMORY\tDISK")
+		_, _ = fmt.Fprintln(w, "PROFILE\tSTATUS\tARCH\tCPUS\tMEMORY\tDISK\tRUNTIME\tADDRESS")
 
 		if len(instances) == 0 {
 			logrus.Warn("No instance found. Run `colima start` to create an instance.")
 		}
 
 		for _, inst := range instances {
-			fmt.Fprintf(w, "%s\t%s\t%s\t%d\t%s\t%s\n",
+			_, _ = fmt.Fprintf(w, "%s\t%s\t%s\t%d\t%s\t%s\t%s\t%s\n",
 				inst.Name,
 				inst.Status,
 				inst.Arch,
 				inst.CPU,
 				units.BytesSize(float64(inst.Memory)),
 				units.BytesSize(float64(inst.Disk)),
+				inst.Runtime,
+				inst.IPAddress,
 			)
 		}
 
@@ -50,4 +70,6 @@ A new instance can be created during 'colima start' by specifying the '--profile
 
 func init() {
 	root.Cmd().AddCommand(listCmd)
+
+	listCmd.Flags().BoolVarP(&listCmdArgs.json, "json", "j", false, "print json output")
 }
