@@ -1,6 +1,8 @@
 package cmd
 
 import (
+	"fmt"
+
 	"github.com/abiosoft/colima/cmd/root"
 	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/environment/vm/lima"
@@ -14,16 +16,22 @@ var sshConfigCmd = &cobra.Command{
 	Long:  `Show configuration of the SSH connection to the VM.`,
 	Args:  cobra.MaximumNArgs(1),
 	RunE: func(cmd *cobra.Command, args []string) error {
-		return lima.ShowSSH(config.Profile().ID, sshConfigCmdArgs.format)
+		resp, err := lima.ShowSSH(config.CurrentProfile().ID, sshConfigCmdArgs.layer, sshConfigCmdArgs.format)
+		if err == nil {
+			fmt.Println(resp.Output)
+		}
+		return err
 	},
 }
 
 var sshConfigCmdArgs struct {
 	format string
+	layer  bool
 }
 
 func init() {
 	root.Cmd().AddCommand(sshConfigCmd)
 
 	sshConfigCmd.Flags().StringVarP(&sshConfigCmdArgs.format, "format", "f", "config", "format (config, cmd)")
+	sshConfigCmd.Flags().BoolVarP(&sshConfigCmdArgs.layer, "layer", "l", true, "config for the Ubuntu layer (if enabled)")
 }
