@@ -1,7 +1,6 @@
 package docker
 
 import (
-	"fmt"
 	"path/filepath"
 
 	"github.com/abiosoft/colima/config"
@@ -9,10 +8,12 @@ import (
 
 // HostSocketFile returns the path to the docker socket on host.
 func HostSocketFile() string { return filepath.Join(config.Dir(), "docker.sock") }
+func LegacyDefaultHostSocketFile() string {
+	return filepath.Join(filepath.Dir(config.Dir()), "docker.sock")
+}
 
 func (d dockerRuntime) isContextCreated() bool {
-	command := fmt.Sprintf(`docker context ls -q | grep "^%s$"`, config.Profile().ID)
-	return d.host.RunQuiet("sh", "-c", command) == nil
+	return d.host.RunQuiet("docker", "context", "inspect", config.CurrentProfile().ID) == nil
 }
 
 func (d dockerRuntime) setupContext() error {
@@ -20,7 +21,7 @@ func (d dockerRuntime) setupContext() error {
 		return nil
 	}
 
-	profile := config.Profile()
+	profile := config.CurrentProfile()
 
 	return d.host.Run("docker", "context", "create", profile.ID,
 		"--description", profile.DisplayName,
@@ -29,7 +30,7 @@ func (d dockerRuntime) setupContext() error {
 }
 
 func (d dockerRuntime) useContext() error {
-	return d.host.Run("docker", "context", "use", config.Profile().ID)
+	return d.host.Run("docker", "context", "use", config.CurrentProfile().ID)
 }
 
 func (d dockerRuntime) teardownContext() error {
@@ -37,5 +38,5 @@ func (d dockerRuntime) teardownContext() error {
 		return nil
 	}
 
-	return d.host.Run("docker", "context", "rm", "--force", config.Profile().ID)
+	return d.host.Run("docker", "context", "rm", "--force", config.CurrentProfile().ID)
 }
