@@ -1,9 +1,11 @@
 package docker
 
 import (
+	"context"
 	"time"
 
 	"github.com/abiosoft/colima/cli"
+	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/environment"
 )
 
@@ -13,7 +15,7 @@ const Name = "docker"
 var _ environment.Container = (*dockerRuntime)(nil)
 
 func init() {
-	environment.RegisterContainer(Name, newRuntime)
+	environment.RegisterContainer(Name, newRuntime, false)
 }
 
 type dockerRuntime struct {
@@ -35,29 +37,20 @@ func (d dockerRuntime) Name() string {
 	return Name
 }
 
-func (d dockerRuntime) isUserPermissionFixed() bool {
-	err := d.guest.RunQuiet("sh", "-c", `getent group docker | grep "\b${USER}\b"`)
-	return err == nil
-}
+func (d dockerRuntime) Provision(ctx context.Context) error {
+	a := d.Init(ctx)
+	log := d.Logger(ctx)
 
-func (d dockerRuntime) Provision() error {
-	a := d.Init()
-	a.Stage("provisioning")
-
-	// check user permission
-	if !d.isUserPermissionFixed() {
-		a.Add(d.fixUserPermission)
-
-		a.Stage("restarting VM to complete setup")
-		a.Add(d.guest.Restart)
-	}
-
-	if !d.isDaemonFileCreated() {
-		a.Add(d.createDaemonFile)
-	}
+	conf, _ := ctx.Value(config.CtxKey()).(config.Config)
 
 	// daemon.json
-	a.Add(d.setupDaemonFile)
+	a.Add(func() error {
+		// not a fatal error
+		if err := d.createDaemonFile(conf.Docker); err != nil {
+			log.Warnln(err)
+		}
+		return nil
+	})
 
 	// docker context
 	a.Add(d.setupContext)
@@ -66,44 +59,45 @@ func (d dockerRuntime) Provision() error {
 	return a.Exec()
 }
 
-func (d dockerRuntime) Start() error {
-	a := d.Init()
-
-	a.Stage("starting")
+func (d dockerRuntime) Start(ctx context.Context) error {
+	a := d.Init(ctx)
 
 	a.Add(func() error {
 		return d.guest.Run("sudo", "service", "docker", "start")
 	})
 
 	// service startup takes few seconds, retry at most 5 times before giving up.
-	a.Retry("waiting for startup to complete", time.Second*5, 10, func() error {
+	a.Retry("", time.Second*5, 12, func(int) error {
 		return d.guest.RunQuiet("sudo", "docker", "info")
 	})
 
 	return a.Exec()
 }
 
-func (d dockerRuntime) Running() bool {
+func (d dockerRuntime) Running(ctx context.Context) bool {
 	return d.guest.RunQuiet("service", "docker", "status") == nil
 }
 
-func (d dockerRuntime) Stop() error {
-	a := d.Init()
-	a.Stage("stopping")
+func (d dockerRuntime) Stop(ctx context.Context) error {
+	a := d.Init(ctx)
 
 	a.Add(func() error {
-		if !d.Running() {
+		if !d.Running(ctx) {
 			return nil
 		}
 		return d.guest.Run("sudo", "service", "docker", "stop")
 	})
 
+	// clear docker context settings
+	// since the container runtime can be changed on startup,
+	// it is better to not leave unnecessary traces behind
+	a.Add(d.teardownContext)
+
 	return a.Exec()
 }
 
-func (d dockerRuntime) Teardown() error {
-	a := d.Init()
-	a.Stage("deleting")
+func (d dockerRuntime) Teardown(ctx context.Context) error {
+	a := d.Init(ctx)
 
 	// clear docker context settings
 	a.Add(d.teardownContext)
@@ -115,7 +109,7 @@ func (d dockerRuntime) Dependencies() []string {
 	return []string{"docker"}
 }
 
-func (d dockerRuntime) Version() string {
+func (d dockerRuntime) Version(ctx context.Context) string {
 	version, _ := d.host.RunOutput("docker", "version", "--format", `client: v{{.Client.Version}}{{printf "\n"}}server: v{{.Server.Version}}`)
 	return version
 }
