@@ -5,22 +5,34 @@ import (
 	"strings"
 
 	"github.com/abiosoft/colima/cli"
+	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/environment"
 	"github.com/abiosoft/colima/environment/container/containerd"
 	"github.com/abiosoft/colima/environment/container/docker"
+	"github.com/abiosoft/colima/environment/vm/lima"
 	"github.com/abiosoft/colima/util/downloader"
 	"github.com/sirupsen/logrus"
 )
 
-const k3sVersion = "v1.22.4+k3s1"
-
-func installK3s(host environment.HostActions, guest environment.GuestActions, a *cli.ActiveCommandChain, log *logrus.Entry, containerRuntime string) {
-	installK3sBinary(host, guest, a)
-	installK3sCache(host, guest, a, log, containerRuntime)
-	installK3sCluster(host, guest, a, containerRuntime)
+func installK3s(host environment.HostActions,
+	guest environment.GuestActions,
+	a *cli.ActiveCommandChain,
+	log *logrus.Entry,
+	containerRuntime string,
+	k3sVersion string,
+	ingress bool,
+) {
+	installK3sBinary(host, guest, a, k3sVersion)
+	installK3sCache(host, guest, a, log, containerRuntime, k3sVersion)
+	installK3sCluster(host, guest, a, containerRuntime, k3sVersion, ingress)
 }
 
-func installK3sBinary(host environment.HostActions, guest environment.GuestActions, a *cli.ActiveCommandChain) {
+func installK3sBinary(
+	host environment.HostActions,
+	guest environment.GuestActions,
+	a *cli.ActiveCommandChain,
+	k3sVersion string,
+) {
 	// install k3s last to ensure it is the last step
 	downloadPath := "/tmp/k3s"
 	url := "https://github.com/k3s-io/k3s/releases/download/" + k3sVersion + "/k3s"
@@ -35,7 +47,14 @@ func installK3sBinary(host environment.HostActions, guest environment.GuestActio
 	})
 }
 
-func installK3sCache(host environment.HostActions, guest environment.GuestActions, a *cli.ActiveCommandChain, log *logrus.Entry, containerRuntime string) {
+func installK3sCache(
+	host environment.HostActions,
+	guest environment.GuestActions,
+	a *cli.ActiveCommandChain,
+	log *logrus.Entry,
+	containerRuntime string,
+	k3sVersion string,
+) {
 	imageTar := "k3s-airgap-images-" + guest.Arch().GoArch() + ".tar"
 	imageTarGz := imageTar + ".gz"
 	downloadPathTar := "/tmp/" + imageTar
@@ -62,7 +81,7 @@ func installK3sCache(host environment.HostActions, guest environment.GuestAction
 	case containerd.Name:
 		a.Stage("loading oci images")
 		a.Add(func() error {
-			if err := guest.Run("sudo", "ctr", "-n", "k8s.io", "images", "import", downloadPathTar); err != nil {
+			if err := guest.Run("sudo", "ctr", "-n", "k8s.io", "images", "import", "--all-platforms", downloadPathTar); err != nil {
 				log.Warnln(fmt.Errorf("error loading oci images: %w", err))
 				log.Warnln("startup may delay a bit as images will be pulled from oci registry")
 			}
@@ -81,7 +100,14 @@ func installK3sCache(host environment.HostActions, guest environment.GuestAction
 
 }
 
-func installK3sCluster(host environment.HostActions, guest environment.GuestActions, a *cli.ActiveCommandChain, containerRuntime string) {
+func installK3sCluster(
+	host environment.HostActions,
+	guest environment.GuestActions,
+	a *cli.ActiveCommandChain,
+	containerRuntime string,
+	k3sVersion string,
+	ingress bool,
+) {
 	// install k3s last to ensure it is the last step
 	downloadPath := "/tmp/k3s-install.sh"
 	url := "https://raw.githubusercontent.com/k3s-io/k3s/" + k3sVersion + "/install.sh"
@@ -95,12 +121,23 @@ func installK3sCluster(host environment.HostActions, guest environment.GuestActi
 	args := []string{
 		"--write-kubeconfig-mode", "644",
 		"--resolv-conf", "/etc/resolv.conf",
-		"--disable", "traefik",
+		"--flannel-iface", "eth0",
+	}
+
+	if !ingress {
+		args = append(args, "--disable", "traefik")
+	}
+
+	// replace ip address if networking is enabled
+	ipAddress := lima.IPAddress(config.CurrentProfile().ID)
+	if ipAddress != "127.0.0.1" {
+		args = append(args, "--bind-address", ipAddress)
+		args = append(args, "--advertise-address", ipAddress)
 	}
 
 	switch containerRuntime {
 	case docker.Name:
-		args = append(args, "--docker")
+		args = append(args, "--container-runtime-endpoint", "unix:///run/cri-dockerd.sock")
 	case containerd.Name:
 		args = append(args, "--container-runtime-endpoint", "unix:///run/containerd/containerd.sock")
 	}
