@@ -1,25 +1,27 @@
 package kubernetes
 
 import (
+	"context"
 	"fmt"
-	"github.com/abiosoft/colima/cli"
-	"github.com/abiosoft/colima/config"
 	"path/filepath"
-	"strconv"
 	"strings"
 	"time"
+
+	"github.com/abiosoft/colima/cli"
+	"github.com/abiosoft/colima/config"
+	"github.com/abiosoft/colima/environment/vm/lima"
 )
 
-const kubeconfigKey = "kubeconfig"
+const masterAddressKey = "master_address"
 
-func (c kubernetesRuntime) provisionKubeconfig() error {
-	provisioned, _ := strconv.ParseBool(c.guest.Get(kubeconfigKey))
-	if provisioned {
+func (c kubernetesRuntime) provisionKubeconfig(ctx context.Context) error {
+	ip := lima.IPAddress(config.CurrentProfile().ID)
+	if ip == c.guest.Get(masterAddressKey) {
 		return nil
 	}
 
-	log := c.Logger()
-	a := c.Init()
+	log := c.Logger(ctx)
+	a := c.Init(ctx)
 
 	a.Stage("updating config")
 
@@ -33,7 +35,7 @@ func (c kubernetesRuntime) provisionKubeconfig() error {
 		return fmt.Errorf("error retrieving home directory on host")
 	}
 
-	profile := config.Profile().ID
+	profile := config.CurrentProfile().ID
 	hostKubeDir := filepath.Join(hostHome, ".kube")
 	a.Add(func() error {
 		return c.host.Run("mkdir", "-p", filepath.Join(hostKubeDir, "."+profile))
@@ -44,7 +46,7 @@ func (c kubernetesRuntime) provisionKubeconfig() error {
 
 	// manipulate in VM and save to host
 	a.Add(func() error {
-		kubeconfig, err := c.guest.RunOutput("cat", "/etc/rancher/k3s/k3s.yaml")
+		kubeconfig, err := c.guest.Read("/etc/rancher/k3s/k3s.yaml")
 		if err != nil {
 			return fmt.Errorf("error fetching kubeconfig on guest: %w", err)
 		}
@@ -100,14 +102,14 @@ func (c kubernetesRuntime) provisionKubeconfig() error {
 
 	// save settings
 	a.Add(func() error {
-		return c.guest.Set(kubeconfigKey, "true")
+		return c.guest.Set(masterAddressKey, ip)
 	})
 
 	return a.Exec()
 }
 
 func (c kubernetesRuntime) unsetKubeconfig(a *cli.ActiveCommandChain) {
-	profile := config.Profile().ID
+	profile := config.CurrentProfile().ID
 	a.Add(func() error {
 		return c.host.Run("kubectl", "config", "unset", "users."+profile)
 	})
@@ -122,4 +124,7 @@ func (c kubernetesRuntime) unsetKubeconfig(a *cli.ActiveCommandChain) {
 func (c kubernetesRuntime) teardownKubeconfig(a *cli.ActiveCommandChain) {
 	a.Stage("reverting config")
 	c.unsetKubeconfig(a)
+	a.Add(func() error {
+		return c.guest.Set(masterAddressKey, "")
+	})
 }
