@@ -1,8 +1,11 @@
 package environment
 
 import (
-	"github.com/abiosoft/colima/config"
+	"context"
+	"io"
 	"os"
+
+	"github.com/abiosoft/colima/config"
 )
 
 type runActions interface {
@@ -15,6 +18,8 @@ type runActions interface {
 	RunOutput(args ...string) (string, error)
 	// RunInteractive runs command interactively.
 	RunInteractive(args ...string) error
+	// RunWith runs with stdin and stdout.
+	RunWith(stdin io.Reader, stdout io.Writer, args ...string) error
 }
 
 type fileActions interface {
@@ -37,16 +42,17 @@ type HostActions interface {
 // GuestActions are actions performed on the guest i.e. VM.
 type GuestActions interface {
 	runActions
+	fileActions
 	// Start starts up the VM
-	Start(config.Config) error
+	Start(ctx context.Context, conf config.Config) error
 	// Stop shuts down the VM
-	Stop() error
+	Stop(ctx context.Context, force bool) error
 	// Restart restarts the VM
-	Restart() error
+	Restart(ctx context.Context) error
 	// Created returns if the VM has been previously created.
 	Created() bool
 	// Running returns if the VM is currently running.
-	Running() bool
+	Running(ctx context.Context) bool
 	// Env retrieves environment variable in the VM.
 	Env(string) (string, error)
 	// Get retrieves a configuration in the VM.
