@@ -1,28 +1,25 @@
 package environment
 
-import "runtime"
+import (
+	"context"
+	"runtime"
+)
 
 // VM is virtual machine.
 type VM interface {
 	GuestActions
 	Dependencies
 	Host() HostActions
-	Teardown() error
+	Teardown(ctx context.Context) error
 }
 
 // VM configurations
 const (
 	// ContainerRuntimeKey is the settings key for container runtime.
 	ContainerRuntimeKey = "runtime"
-	// KubernetesVersionKey is the settings key for kubernetes version.
-	KubernetesVersionKey = "kubernetes_version"
-
-	// BinfmtTarFile is the path in the VM to the binfmt oci image tar.
-	// TODO: decide if this should reside somewhere else.
-	BinfmtTarFile = "/usr/local/colima/binfmt.tar"
 )
 
-// Arch is the VM architecture.
+// Arch is the CPU architecture of the VM.
 type Arch string
 
 const (
@@ -30,6 +27,11 @@ const (
 	AARCH64 Arch = "aarch64"
 )
 
+// HostArch returns the host CPU architecture.
+func HostArch() Arch {
+	return Arch(runtime.GOARCH)
+}
+
 // GoArch returns the GOARCH equivalent value for the architecture.
 func (a Arch) GoArch() string {
 	switch a {
@@ -54,5 +56,5 @@ func (a Arch) Value() Arch {
 		return AARCH64
 	}
 
-	return "default"
+	return Arch(runtime.GOARCH).Value()
 }
