@@ -1,15 +1,18 @@
 package lima
 
 import (
+	"context"
 	"fmt"
+	"net"
 	"path/filepath"
+	"strconv"
 
 	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/util"
 )
 
 func (l limaVM) copyCerts() error {
-	log := l.Logger()
+	log := l.Logger(context.Background())
 	err := func() error {
 		dockerCertsDirHost := filepath.Join(util.HomeDir(), ".docker", "certs.d")
 		dockerCertsDirGuest := "/etc/docker/certs.d"
@@ -42,3 +45,9 @@ func (l limaVM) copyCerts() error {
 	}
 	return nil
 }
+
+func (l limaVM) addHost(host string, ip net.IP) error {
+	line := fmt.Sprintf("%s\t%s", ip.String(), host)
+	line = fmt.Sprintf("echo -e %s >> /etc/hosts", strconv.Quote(line))
+	return l.Run("sudo", "sh", "-c", line)
+}
