@@ -1,19 +1,23 @@
 package lima
 
 import (
-	"bufio"
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
+	"io"
+	"net"
 	"os"
 	"path/filepath"
-	"strconv"
-	"strings"
 	"time"
 
+	"github.com/abiosoft/colima/environment/vm/lima/network/daemon/gvproxy"
+	"github.com/abiosoft/colima/environment/vm/lima/network/daemon/vmnet"
+
 	"github.com/abiosoft/colima/cli"
 	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/environment"
+	"github.com/abiosoft/colima/environment/vm/lima/network"
 	"github.com/abiosoft/colima/util"
 	"github.com/abiosoft/colima/util/yamlutil"
 	"github.com/sirupsen/logrus"
@@ -21,7 +25,13 @@ import (
 
 // New creates a new virtual machine.
 func New(host environment.HostActions) environment.VM {
-	env := limaInstanceEnvVar + "=" + config.Profile().ID
+	var envs []string
+	envLimaInstance := limaInstanceEnvVar + "=" + config.CurrentProfile().ID
+	envSubprocess := config.SubprocessProfileEnvVar + "=" + config.CurrentProfile().ShortName
+	envs = append(envs, envLimaInstance, envSubprocess)
+
+	binDir := filepath.Join(config.WrapperDir(), "bin")
+	envs = append(envs, "PATH="+util.AppendToPath(os.Getenv("PATH"), binDir))
 
 	home, err := limaHome()
 	if err != nil {
@@ -33,9 +43,10 @@ func New(host environment.HostActions) environment.VM {
 
 	// consider making this truly flexible to support other VMs
 	return &limaVM{
-		host:         host.WithEnv(env),
+		host:         host.WithEnv(envs...),
 		home:         home,
 		CommandChain: cli.New("vm"),
+		network:      network.NewManager(host),
 	}
 }
 
@@ -43,6 +54,7 @@ const (
 	limaInstanceEnvVar = "LIMA_INSTANCE"
 	lima               = "lima"
 	limactl            = "limactl"
+	layerEnvVar        = "COLIMA_LAYER_SSH_PORT"
 )
 
 func limaHome() (string, error) {
@@ -68,7 +80,7 @@ func limaHome() (string, error) {
 }
 
 func (l limaVM) limaConfDir() string {
-	return filepath.Join(l.home, config.Profile().ID)
+	return filepath.Join(l.home, config.CurrentProfile().ID)
 }
 
 var _ environment.VM = (*limaVM)(nil)
@@ -82,6 +94,9 @@ type limaVM struct {
 
 	// lima config directory
 	home string
+
+	// network between host and the vm
+	network network.Manager
 }
 
 func (l limaVM) Dependencies() []string {
@@ -90,19 +105,110 @@ func (l limaVM) Dependencies() []string {
 	}
 }
 
-func (l *limaVM) Start(conf config.Config) error {
-	a := l.Init()
-
-	if l.Created() {
-		return l.resume(conf)
+func (l *limaVM) prepareNetwork(ctx context.Context, conf config.Network) (context.Context, error) {
+	// limited to macOS for now
+	if !util.MacOS() {
+		return ctx, nil
 	}
 
-	a.Stage("creating and starting")
+	ctxKeyVmnet := network.CtxKey(vmnet.Name())
+	ctxKeyGVProxy := network.CtxKey(gvproxy.Name())
 
-	configFile := config.Profile().ID + ".yaml"
+	// use a nested chain for convenience
+	a := l.Init(ctx)
+	log := l.Logger(ctx)
+
+	a.Stage("preparing network")
+	a.Add(func() error {
+		ctx = context.WithValue(ctx, ctxKeyGVProxy, true)
+		if conf.Address {
+			ctx = context.WithValue(ctx, ctxKeyVmnet, true)
+		}
+		deps, root := l.network.Dependencies(ctx)
+		if deps.Installed() {
+			return nil
+		}
+
+		// if user interaction is not required (i.e. root),
+		// no need for another verbose info.
+		if root {
+			log.Println("dependencies missing for setting up reachable IP address")
+			log.Println("sudo password may be required")
+		}
+		return deps.Install(l.host)
+	})
 
 	a.Add(func() error {
-		limaConf, err := newConf(conf)
+		return l.network.Start(ctx)
+	})
+
+	// delay to ensure that the vmnet is running
+	statusKey := "networkStatus"
+	if conf.Address {
+		a.Retry("", time.Second*3, 5, func(i int) error {
+			s, err := l.network.Running(ctx)
+			ctx = context.WithValue(ctx, statusKey, s)
+			if err != nil {
+				return err
+			}
+			if !s.Running {
+				return fmt.Errorf("network process is not running")
+			}
+			for _, p := range s.Processes {
+				if !p.Running {
+					return p.Error
+				}
+			}
+			return nil
+		})
+	}
+
+	// network failure is not fatal
+	if err := a.Exec(); err != nil {
+		func() {
+			status, ok := ctx.Value(statusKey).(network.Status)
+			if !ok {
+				return
+			}
+			if !status.Running {
+				log.Warnln(fmt.Errorf("error starting network: %w", err))
+				return
+			}
+
+			for _, p := range status.Processes {
+				if !p.Running {
+					ctx = context.WithValue(ctx, network.CtxKey(p.Name), false)
+					log.Warnln(fmt.Errorf("error starting %s: %w", p.Name, err))
+				}
+			}
+		}()
+	}
+
+	// preserve gvproxy context
+	if gvproxyEnabled, _ := ctx.Value(network.CtxKey(gvproxy.Name())).(bool); gvproxyEnabled {
+		l.host = l.host.WithEnv(gvproxy.SubProcessEnvVar + "=1")
+	}
+
+	return ctx, nil
+}
+
+func (l *limaVM) Start(ctx context.Context, conf config.Config) error {
+	a := l.Init(ctx)
+
+	if l.Created() {
+		return l.resume(ctx, conf)
+	}
+
+	a.Add(func() (err error) {
+		ctx, err = l.prepareNetwork(ctx, conf.Network)
+		return err
+	})
+
+	a.Stage("creating and starting")
+	configFile := filepath.Join(os.TempDir(), config.CurrentProfile().ID+".yaml")
+
+	a.Add(func() error {
+		limaConf, err := newConf(ctx, conf)
 		if err != nil {
 			return err
 		}
@@ -115,11 +221,21 @@ func (l *limaVM) Start(conf config.Config) error {
 		return os.Remove(configFile)
 	})
 
+	// host file
+	{
+		// add docker host alias
+		a.Add(func() error {
+			return l.addHost("host.docker.internal", net.ParseIP("192.168.5.2"))
+		})
+		// prevent chroot host error for layer
+		a.Add(func() error {
+			return l.addHost(config.CurrentProfile().ID, net.ParseIP("127.0.0.1"))
+		})
+	}
+
 	// registry certs
 	a.Add(l.copyCerts)
 
-	l.applyDNS(a, conf)
-
 	// adding it to command chain to execute only after successful startup.
 	a.Add(func() error {
 		l.conf = conf
@@ -129,19 +245,24 @@ func (l *limaVM) Start(conf config.Config) error {
 	return a.Exec()
 }
 
-func (l limaVM) resume(conf config.Config) error {
-	log := l.Logger()
-	a := l.Init()
+func (l limaVM) resume(ctx context.Context, conf config.Config) error {
+	log := l.Logger(ctx)
+	a := l.Init(ctx)
 
-	if l.Running() {
+	if l.Running(ctx) {
 		log.Println("already running")
 		return nil
 	}
 
+	a.Add(func() (err error) {
+		ctx, err = l.prepareNetwork(ctx, conf.Network)
+		return err
+	})
+
 	configFile := filepath.Join(l.limaConfDir(), "lima.yaml")
 
 	a.Add(func() error {
-		limaConf, err := newConf(conf)
+		limaConf, err := newConf(ctx, conf)
 		if err != nil {
 			return err
 		}
@@ -150,110 +271,79 @@ func (l limaVM) resume(conf config.Config) error {
 
 	a.Stage("starting")
 	a.Add(func() error {
-		return l.host.Run(limactl, "start", config.Profile().ID)
+		return l.host.Run(limactl, "start", config.CurrentProfile().ID)
 	})
 
 	// registry certs
 	a.Add(l.copyCerts)
 
-	l.applyDNS(a, conf)
-
 	return a.Exec()
 }
 
-func (l limaVM) applyDNS(a *cli.ActiveCommandChain, conf config.Config) {
-	// manually set the DNS by modifying the resolve file.
-	//
-	// Lima's DNS settings is fixed at VM create and cannot be changed afterwards.
-	// this is a better approach as it only applies on VM startup and gets reset at shutdown.
-	// this is specific to Alpine , may be different for other distros.
-	log := l.Logger()
-	dnsFile := "/etc/resolv.conf"
-	dnsFileBak := dnsFile + ".lima"
-
-	a.Add(func() error {
-		// backup the original dns file (if not previously done)
-		if l.RunQuiet("stat", dnsFileBak) != nil {
-			err := l.RunQuiet("sudo", "cp", dnsFile, dnsFileBak)
-			if err != nil {
-				// custom DNS config failure should not prevent the VM from starting
-				// as the default config will be used.
-				// Rather, warn and terminate setting the DNS config.
-				log.Warnln(fmt.Errorf("error backing up default DNS config: %w", err))
-				return nil
-			}
-		}
-		return nil
-	})
-
-	a.Add(func() error {
-		// empty the file
-		if err := l.RunQuiet("sudo", "rm", "-f", dnsFile); err != nil {
-			return fmt.Errorf("error initiating DNS config: %w", err)
-		}
-
-		for _, dns := range conf.VM.DNS {
-			line := fmt.Sprintf(`echo nameserver %s >> %s`, dns.String(), dnsFile)
-			if err := l.RunQuiet("sudo", "sh", "-c", line); err != nil {
-				return fmt.Errorf("error applying DNS config: %w", err)
-			}
-		}
-
-		if len(conf.VM.DNS) > 0 {
-			return nil
-		}
-
-		// use the default Lima dns if no dns is set
-		return l.RunQuiet("sudo", "sh", "-c", fmt.Sprintf("cat %s >> %s", dnsFileBak, dnsFile))
-	})
+func (l limaVM) Running(ctx context.Context) bool {
+	i, err := Instance()
+	if err != nil {
+		logrus.Trace(fmt.Errorf("error retrieving running instance: %w", err))
+		return false
+	}
+	return i.Running()
 }
 
-func (l limaVM) Running() bool {
-	return l.RunQuiet("uname") == nil
-}
-
-func (l limaVM) Stop() error {
-	log := l.Logger()
-	a := l.Init()
-	if !l.Running() {
+func (l limaVM) Stop(ctx context.Context, force bool) error {
+	log := l.Logger(ctx)
+	a := l.Init(ctx)
+	if !l.Running(ctx) {
 		log.Println("not running")
 		return nil
 	}
 
 	a.Stage("stopping")
 
+	if util.MacOS() {
+		a.Retry("", time.Second*1, 10, func(retryCount int) error {
+			return l.network.Stop(ctx)
+		})
+	}
+
 	a.Add(func() error {
-		return l.host.Run(limactl, "stop", config.Profile().ID)
+		if force {
+			return l.host.Run(limactl, "stop", "--force", config.CurrentProfile().ID)
+		}
+		return l.host.Run(limactl, "stop", config.CurrentProfile().ID)
 	})
 
 	return a.Exec()
 }
 
-func (l limaVM) Teardown() error {
-	a := l.Init()
+func (l limaVM) Teardown(ctx context.Context) error {
+	a := l.Init(ctx)
 
-	a.Stage("deleting")
+	if util.MacOS() {
+		a.Retry("", time.Second*1, 10, func(retryCount int) error {
+			return l.network.Stop(ctx)
+		})
+	}
 
 	a.Add(func() error {
-		return l.host.Run(limactl, "delete", "--force", config.Profile().ID)
+		return l.host.Run(limactl, "delete", "--force", config.CurrentProfile().ID)
 	})
 
 	return a.Exec()
 }
 
-func (l limaVM) Restart() error {
+func (l limaVM) Restart(ctx context.Context) error {
 	if l.conf.Empty() {
 		return fmt.Errorf("cannot restart, VM not previously started")
 	}
 
-	if err := l.Stop(); err != nil {
+	if err := l.Stop(ctx, false); err != nil {
 		return err
 	}
 
 	// minor delay to prevent possible race condition.
 	time.Sleep(time.Second * 2)
 
-	if err := l.Start(l.conf); err != nil {
+	if err := l.Start(ctx, l.conf); err != nil {
 		return err
 	}
 
@@ -263,7 +353,7 @@ func (l limaVM) Restart() error {
 func (l limaVM) Run(args ...string) error {
 	args = append([]string{lima}, args...)
 
-	a := l.Init()
+	a := l.Init(context.Background())
 
 	a.Add(func() error {
 		return l.host.Run(args...)
@@ -275,7 +365,7 @@ func (l limaVM) Run(args ...string) error {
 func (l limaVM) RunInteractive(args ...string) error {
 	args = append([]string{lima}, args...)
 
-	a := l.Init()
+	a := l.Init(context.Background())
 
 	a.Add(func() error {
 		return l.host.RunInteractive(args...)
@@ -284,10 +374,22 @@ func (l limaVM) RunInteractive(args ...string) error {
 	return a.Exec()
 }
 
+func (l limaVM) RunWith(stdin io.Reader, stdout io.Writer, args ...string) error {
+	args = append([]string{lima}, args...)
+
+	a := l.Init(context.Background())
+
+	a.Add(func() error {
+		return l.host.RunWith(stdin, stdout, args...)
+	})
+
+	return a.Exec()
+}
+
 func (l limaVM) RunOutput(args ...string) (out string, err error) {
 	args = append([]string{lima}, args...)
 
-	a := l.Init()
+	a := l.Init(context.Background())
 
 	a.Add(func() (err error) {
 		out, err = l.host.RunOutput(args...)
@@ -301,7 +403,7 @@ func (l limaVM) RunOutput(args ...string) (out string, err error) {
 func (l limaVM) RunQuiet(args ...string) (err error) {
 	args = append([]string{lima}, args...)
 
-	a := l.Init()
+	a := l.Init(context.Background())
 
 	a.Add(func() (err error) {
 		return l.host.RunQuiet(args...)
@@ -316,7 +418,8 @@ func (l limaVM) Host() environment.HostActions {
 }
 
 func (l limaVM) Env(s string) (string, error) {
-	if !l.Running() {
+	ctx := context.Background()
+	if !l.Running(ctx) {
 		return "", fmt.Errorf("not running")
 	}
 	return l.RunOutput("echo", "$"+s)
@@ -331,7 +434,7 @@ const configFile = "/etc/colima/colima.json"
 
 func (l limaVM) getConf() map[string]string {
 	obj := map[string]string{}
-	b, err := l.RunOutput("cat", configFile)
+	b, err := l.Read(configFile)
 	if err != nil {
 		return obj
 	}
@@ -361,7 +464,8 @@ func (l limaVM) Set(key, value string) error {
 	if err := l.Run("sudo", "mkdir", "-p", filepath.Dir(configFile)); err != nil {
 		return fmt.Errorf("error saving settings: %w", err)
 	}
-	if err := l.Run("sudo", "sh", "-c", fmt.Sprintf(`echo %s > %s`, strconv.Quote(string(b)), configFile)); err != nil {
+
+	if err := l.Write(configFile, string(b)); err != nil {
 		return fmt.Errorf("error saving settings: %w", err)
 	}
 
@@ -376,73 +480,3 @@ func (l limaVM) Arch() environment.Arch {
 	a, _ := l.RunOutput("uname", "-m")
 	return environment.Arch(a)
 }
-
-// InstanceInfo is the information about a Lima instance
-type InstanceInfo struct {
-	Name   string `json:"name,omitempty"`
-	Status string `json:"status,omitempty"`
-	Arch   string `json:"arch,omitempty"`
-	CPU    int    `json:"cpus,omitempty"`
-	Memory int64  `json:"memory,omitempty"`
-	Disk   int64  `json:"disk,omitempty"`
-}
-
-// Instances returns Lima instances created by colima.
-func Instances() ([]InstanceInfo, error) {
-	var buf bytes.Buffer
-	cmd := cli.Command("limactl", "list", "--json")
-	cmd.Stdout = &buf
-
-	if err := cmd.Run(); err != nil {
-		return nil, fmt.Errorf("error retrieving instances: %w", err)
-	}
-
-	var instances []InstanceInfo
-	scanner := bufio.NewScanner(&buf)
-	for scanner.Scan() {
-		var i InstanceInfo
-		line := scanner.Bytes()
-		if err := json.Unmarshal(line, &i); err != nil {
-			return nil, fmt.Errorf("error retrieving instances: %w", err)
-		}
-
-		// limit to colima instances
-		if !strings.HasPrefix(i.Name, "colima") {
-			continue
-		}
-
-		// rename to local friendly names
-		i.Name = toUserFriendlyName(i.Name)
-
-		instances = append(instances, i)
-	}
-
-	return instances, nil
-}
-
-// ShowSSH runs the show-ssh command in Lima.
-func ShowSSH(name, format string) error {
-	var buf bytes.Buffer
-	cmd := cli.Command("limactl", "show-ssh", "--format", format, name)
-	cmd.Stdout = &buf
-	cmd.Stderr = os.Stderr
-
-	if err := cmd.Run(); err != nil {
-		return fmt.Errorf("error retrieving ssh config: %w", err)
-	}
-
-	// TODO: this is a lazy approach, edge cases may not be covered
-	from := "lima-" + name
-	to := name
-	out := strings.ReplaceAll(buf.String(), from, to)
-
-	fmt.Println(out)
-	return nil
-}
-
-func toUserFriendlyName(name string) string {
-	if name == "colima" {
-		name = "default"
-	}
-	return strings.TrimPrefix(name, "colima-")
-}
